/*globals window, define*/
/*jslint nomen:true, unparam: true*/

/*
 * @copyright       © Copyright 2006-2013, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages/customform/inc/customformaddfielddialog
 */

define("de_epages/customform/inc/customformaddfielddialog", [
    "jquery",
    "ep",
    "$dict!../dictionary",
    "$tmpl!./customformaddfielddialog-add",
    "$tmpl!./customformaddfielddialog-fieldpreview",

    "ep/ui/dialog",
    "ep/ui/validate",
    "ep/validate"
], function ($, ep, customformDict, tmplCustomformAdd, tmplCustomformFieldpreview) {
    'use strict';

    /*
     * @dictionary      ep.dict
     *
     * @translation     {Type}
     *                              {MandatoryStar}
     *                              {yes}
     *                              {no}
     *                              {Apply}
     *                              {Cancel}
     *
     *
     * @dictionary      de_epages.customform.dictionary
     *
     * @translation     {FieldName}
     *                              {MandatoryForUsers}
     *                              {AdditionalFormFieldText}
     *                              {AddFormField}
     */

    return {
        customformGenerateAddDialog: function (options) {

            var o = $.extend({
//                  ViewAction: undefined,
//                  ObjectID: undefined,
//                  element: undefined,
//                  IsEditorMode: undefined,
                    width: 800,
                    height: 400,
                    dialogContentId: 'addDialogContent',
                    selectOptions: {},
                    ChangeAction: 'AddField',
                    languageId: 1
                }, options),

                // translations
                translationsDialog = {
                    tType: customformDict.translate('Type'),
                    tTitle: customformDict.translate('AddFormField'),
                    tFieldName: customformDict.translate('FieldName'),
                    tMandatoryStar: customformDict.translate('MandatoryStar'),
                    tMandatoryForUsers: customformDict.translate('MandatoryForUsers'),
                    tYes: customformDict.translate('yes'),
                    tNo: customformDict.translate('no'),
                    tAdditionalFormFieldText: customformDict.translate('AdditionalFormFieldText'),
                    tAdd: customformDict.translate('Apply'),
                    tCancel: customformDict.translate('Cancel')
                },

                addDialog,
                // identify if plugin was called from preview view
                isEditorMode = (o.IsEditorMode === undefined) ? '' : '&IsEditorMode=' + o.IsEditorMode,

                // initialize
                init = function () {
                    // if an element is defined
                    if (o.element !== undefined) {

                        o.element.on('click', function (event) {
                            event.preventDefault();

                            // remove already existing dialog
                            if ($('#' + o.dialogContentId).length) {
                                $('#' + o.dialogContentId).remove();
                            }
                            // append necessary template to document
                            tmplCustomformAdd($.extend({options: o.selectOptions, LangID: o.languageId}, translationsDialog)).appendTo('body');
                            $('#' + o.dialogContentId + ' input, #' + o.dialogContentId + ' select').metaparse();

                            // create dialog
                            addDialog = ep('#' + o.dialogContentId).uiDialog({
                                title: translationsDialog.tTitle,
                                modal: true,
                                width: o.width,
                                height: o.height,
                                form: {
                                    action: '?ChangeAction=' + o.ChangeAction + '&ViewAction=' + o.ViewAction + '&ObjectID=' + o.ObjectID + isEditorMode,
                                    method: 'post'
                                },
                                open: function () {
                                    var dialog = $(this),
                                        previewFieldType = dialog.find('.previewFieldType'),            // dom element for the preview template
                                        aliasFieldType = dialog.find('.aliasFieldType'),                    // get dom element for the choosen field alias
                                        fieldSelect = dialog.find('select[name=FieldClassID]');     // select for field type;

                                    // handling for field type select box
                                    fieldSelect
                                        .on('change', function () {
                                            var selectedOption = $(this).find('option:selected'),
                                                optionAlias = selectedOption.data('alias'),
                                                mandatoryStar = dialog.find("span.MandatoryStar"),
                                                additionalHelpTextWrapper = dialog.find("input[name=AdditionalHelpText]").closest('div');

                                            // update type element
                                            aliasFieldType.html(translationsDialog.tType + ": " + selectedOption.text());

                                            // remove class from the latest choosen field type
                                            if (previewFieldType.data('lastclass') !== undefined) {
                                                previewFieldType.removeClass(previewFieldType.data('lastclass'));
                                            }

                                            // set/unset mandatory field (if choosen element is no caption field)
                                            dialog.find(".customFormMandatoryElement, .customFormAdditionalTextElement").css('display', ((optionAlias === "CaptionFormField") ? 'none' : ''));

                                            previewFieldType
                                                .addClass(optionAlias)
                                                .data('lastclass', optionAlias);

                                            if (optionAlias === "CaptionFormField") {
                                                mandatoryStar.css('visibility', 'hidden');
                                                additionalHelpTextWrapper.hide();
                                                ep(dialog.find("input[name=FieldName]"))
                                                    .uiValidate({required: false})
                                                    .metaparse();
                                            } else {
                                                mandatoryStar.css('visibility', 'visible');
                                                additionalHelpTextWrapper.show();
                                                ep(dialog.find("input[name=FieldName]"))
                                                    .uiValidate({required: true})
                                                    .metaparse();
                                            }

                                            tmplCustomformFieldpreview({fieldType: optionAlias, description: selectedOption.data('description')}).appendTo(previewFieldType.empty());
                                        })
                                        .trigger('change');
                                },
                                buttons: [
                                    {text: translationsDialog.tAdd, type: 'submit'},
                                    {text: translationsDialog.tCancel, click: function () { ep(this).uiDialog('close'); }}
                                ]
                            });
                        });
                    }
                };

            init();
        },

        // delete form field
        customformDeleteField: function (selector) {
            $(selector).on('click', function (evt) {
                var _this = $(this),
                    form = $(this).closest('form'),
                    hidden = $('<input type="hidden" name="DeleteObjectID" value="' + _this.data('objectid') + '" />');

                // fade out parent table row
                _this.closest('tr').fadeOut({
                    complete: function () {
                        // trigger change event to activate save button
                        form.trigger('change');
                    }
                });

                // add hidden input containing delete information (send and deleted when "Save" is pressed)
                hidden.appendTo(form);
            });
        }
    };

});
